#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <pthread.h>

#include "stock.h"
#include "threads.h"

/* this file is meant to define utility functions, 
 * mainly stuff like malloc extenders, signal and thread initialisation code.
 * and error codes and reporting...
 */

/* error messages array. */

/* These should be read from a file. 

char *error_funny[] = {
   "Sorry, I cannot create server sockets for listening, no game for you. Go Home and study.",
   "I had trouble looking for the initialisation file : company.dat",
   "I cannot accept any connections. I think I am full now.",
   "Not enough system resources to create a new thread. Are you sure you have enough yarn ??",
   "Oh My God, malloc screwed up, can't allocate memory, We'll all die",
   "I can't resize my memory, am i getting mentally retarded or what ?",
   "I can't create the file to store the company info. Don't you like me ?"
}

 */

inline void log_error (int err) {
   fprintf (logfile, "%d\n", err);
}

/* malloc, realloc extenders, so we don't have to check the return 
 * values everytime and the behaviour can be easily changed
 */

void *xmalloc(size_t size) {
   void *xptr;
   
   if ((xptr = malloc(size)) != NULL) return xptr;
   else {
      register int i=1;
      while (1) {
	 sleep (i+i);
#ifdef DEBUG
	 printf ("malloc trouble\n");
#endif
	 
#ifdef LOG
	 fprintf (logfile, "malloc trouble\n");
#endif
	 if ((xptr = malloc(size)) != NULL) return xptr;
	 ++i;
      }
   }
}


void *xrealloc(void *xptr, size_t size) {
   
   if ((xptr = realloc(xptr, size)) != NULL) return xptr;
   else {
      register int i=1;
      while (1) {
	 sleep (i+i);
#ifdef DEBUG
	 printf ("realloc trouble\n");
#endif
	 
#ifdef LOG
	 fprintf (logfile, "realloc trouble\n");
#endif

	 if ((xptr = realloc(xptr, size)) != NULL) return xptr;
      }
   }
}


/* Ignore SIGPIPE, so that EPIPE can be checked for. This behaviour is defined
 * here so that all threads inherit the same behaviour. It is used to 
 * check if a write is done on a closed socket and take appropriate action. 
 * attach handler to SIGALRM... to clear result queues..
 */

static void quit (int signum) {
   pthread_kill_other_threads_np ();
   fprintf (logfile, "Got Ctrl-C\n");   
   fclose (logfile);
   _exit (4);
}

static void punish_violation (int signum) {
   sleep (punish_interval);
}

void signal_stuff () {
   
   signal (SIGPIPE, SIG_IGN);
   signal (SIGALRM, punish_violation);
   signal (SIGINT, quit);
}

/* Thread initialisation stuff.. set default thread attributes and TSD keys
 * initialise mutexes 
 */

void thread_stuff () {

   pthread_attr_init (&attr_detached);   
   pthread_attr_setdetachstate (&attr_detached, PTHREAD_CREATE_DETACHED);
   
   pthread_setcanceltype (PTHREAD_CANCEL_DEFERRED, NULL);
   pthread_setcancelstate (PTHREAD_CANCEL_ENABLE, NULL);
   
   pthread_key_create (&plid_TSD, NULL);
   pthread_key_create (&id_TSD, NULL);
  
   pthread_mutex_init (&req_list_mut, NULL);
}
